{===============================================================================
  RzBizCmp Unit

  This unit implements the classes which form the foundation for the Raize
  Business Component architecture.  The classes defined here are used for
  building data-aware business components.

  Copyright  1995-1996 by Raize Software Solutions, Inc.
===============================================================================}

unit RzBizCmp;

interface

uses
  Classes, Forms, SysUtils, DB, DBTables, RzErrHnd;

type
  EInvalidParameters = class( Exception );
  EValidationError = class( EAbort );

  {== Internal event types handled by TRzBusinessComponent ==}

  TDataEventType = ( etStartEdit, etFinishEdit,
                     etStartInsert, etFinishInsert,
                     etStartDelete, etFinishDelete,
                     etStartPost, etFinishPost,
                     etStartCancel, etFinishCancel,
                     etCalculateFields );

  TBusinessDataEvent = procedure( EventType : TDataEventType ) of object;
  TKeyViolationEvent = procedure( const TableName : string ) of object;

  {========================================}
  {== TRzBusinessTable Class Declaration ==}
  {========================================}

  TRzBusinessTable = class( TTable )
  private
    FOnStartEdit : TBusinessDataEvent;
    FOnFinishEdit : TBusinessDataEvent;
    FOnStartInsert : TBusinessDataEvent;
    FOnFinishInsert : TBusinessDataEvent;
    FOnStartDelete : TBusinessDataEvent;
    FOnFinishDelete : TBusinessDataEvent;
    FOnStartPost : TBusinessDataEvent;
    FOnFinishPost : TBusinessDataEvent;
    FOnStartCancel : TBusinessDataEvent;
    FOnFinishCancel : TBusinessDataEvent;
    FOnCalculateFields : TBusinessDataEvent;
    FOnKeyViolation : TKeyViolationEvent;
    FOldExceptionHandler : TExceptionEvent;
  protected
    procedure DoStartEdit; virtual;
    procedure DoBeforeEdit; override;
    procedure DoAfterEdit; override;
    procedure DoFinishEdit; virtual;

    procedure DoStartInsert; virtual;
    procedure DoBeforeInsert; override;
    procedure DoAfterInsert; override;
    procedure DoFinishInsert; virtual;

    procedure DoStartDelete; virtual;
    procedure DoBeforeDelete; override;
    procedure DoAfterDelete; override;
    procedure DoFinishDelete; virtual;

    procedure DoStartPost; virtual;
    procedure DoBeforePost; override;
    procedure DoAfterPost; override;
    procedure DoFinishPost; virtual;

    procedure DoStartCancel; virtual;
    procedure DoBeforeCancel; override;
    procedure DoAfterCancel; override;
    procedure DoFinishCancel; virtual;

    procedure DoCalculateFields; virtual;
    procedure DoOnCalcFields; override;

    procedure ExceptionHandlerOnPost( Sender : TObject; E : Exception );

    property OnStartEdit : TBusinessDataEvent
      read FOnStartEdit
      write FOnStartEdit;

    property OnFinishEdit : TBusinessDataEvent
      read FOnFinishEdit
      write FOnFinishEdit;

    property OnStartInsert : TBusinessDataEvent
      read FOnStartInsert
      write FOnStartInsert;

    property OnFinishInsert : TBusinessDataEvent
      read FOnFinishInsert
      write FOnFinishInsert;

    property OnStartDelete : TBusinessDataEvent
      read FOnStartDelete
      write FOnStartDelete;

    property OnFinishDelete : TBusinessDataEvent
      read FOnFinishDelete
      write FOnFinishDelete;

    property OnStartPost : TBusinessDataEvent
      read FOnStartPost
      write FOnStartPost;

    property OnFinishPost : TBusinessDataEvent
      read FOnFinishPost
      write FOnFinishPost;

    property OnStartCancel : TBusinessDataEvent
      read FOnStartCancel
      write FOnStartCancel;

    property OnFinishCancel : TBusinessDataEvent
      read FOnFinishCancel
      write FOnFinishCancel;

    property OnCalculateFields : TBusinessDataEvent
      read FOnCalculateFields
      write FOnCalculateFields;

    property OnKeyViolation : TKeyViolationEvent
      read FOnKeyViolation
      write FOnKeyViolation;
  end; {== TRzBusinessTable ==}


  {========================================}
  {== TRzBusinessQuery Class Declaration ==}
  {========================================}

  TRzBusinessQuery = class( TQuery )
  private
    FOnStartEdit : TBusinessDataEvent;
    FOnFinishEdit : TBusinessDataEvent;
    FOnStartInsert : TBusinessDataEvent;
    FOnFinishInsert : TBusinessDataEvent;
    FOnStartDelete : TBusinessDataEvent;
    FOnFinishDelete : TBusinessDataEvent;
    FOnStartPost : TBusinessDataEvent;
    FOnFinishPost : TBusinessDataEvent;
    FOnStartCancel : TBusinessDataEvent;
    FOnFinishCancel : TBusinessDataEvent;
    FOnCalculateFields : TBusinessDataEvent;
    FOnKeyViolation : TKeyViolationEvent;
    FOldExceptionHandler : TExceptionEvent;
  protected
    procedure DoStartEdit; virtual;
    procedure DoBeforeEdit; override;
    procedure DoAfterEdit; override;
    procedure DoFinishEdit; virtual;

    procedure DoStartInsert; virtual;
    procedure DoBeforeInsert; override;
    procedure DoAfterInsert; override;
    procedure DoFinishInsert; virtual;

    procedure DoStartDelete; virtual;
    procedure DoBeforeDelete; override;
    procedure DoAfterDelete; override;
    procedure DoFinishDelete; virtual;

    procedure DoStartPost; virtual;
    procedure DoBeforePost; override;
    procedure DoAfterPost; override;
    procedure DoFinishPost; virtual;

    procedure DoStartCancel; virtual;
    procedure DoBeforeCancel; override;
    procedure DoAfterCancel; override;
    procedure DoFinishCancel; virtual;

    procedure DoCalculateFields; virtual;
    procedure DoOnCalcFields; override;

    procedure ExceptionHandlerOnPost( Sender : TObject; E : Exception );

    property OnStartEdit : TBusinessDataEvent
      read FOnStartEdit
      write FOnStartEdit;

    property OnFinishEdit : TBusinessDataEvent
      read FOnFinishEdit
      write FOnFinishEdit;

    property OnStartInsert : TBusinessDataEvent
      read FOnStartInsert
      write FOnStartInsert;

    property OnFinishInsert : TBusinessDataEvent
      read FOnFinishInsert
      write FOnFinishInsert;

    property OnStartDelete : TBusinessDataEvent
      read FOnStartDelete
      write FOnStartDelete;

    property OnFinishDelete : TBusinessDataEvent
      read FOnFinishDelete
      write FOnFinishDelete;

    property OnStartPost : TBusinessDataEvent
      read FOnStartPost
      write FOnStartPost;

    property OnFinishPost : TBusinessDataEvent
      read FOnFinishPost
      write FOnFinishPost;

    property OnStartCancel : TBusinessDataEvent
      read FOnStartCancel
      write FOnStartCancel;

    property OnFinishCancel : TBusinessDataEvent
      read FOnFinishCancel
      write FOnFinishCancel;

    property OnCalculateFields : TBusinessDataEvent
      read FOnCalculateFields
      write FOnCalculateFields;

    property OnKeyViolation : TKeyViolationEvent
      read FOnKeyViolation
      write FOnKeyViolation;
  end; {== TRzBusinessQuery ==}


  {============================================}
  {== TRzBusinessComponent Class Declaration ==}
  {============================================}

  TRzBusinessComponent = class( TComponent )
  private
    FMainDataset : TDataset;
    FErrorHandler : TRzErrorHandler;
    FSkipValidation : Boolean;
    function  GetIndexName : string;
    procedure SetIndexName( const Value : string );
  protected
    procedure Notification( AComponent : TComponent;
                            Operation : TOperation ); override;
    procedure SetEmbeddedObjectsDataset; virtual;
    procedure LinkDataEvents( Dataset : TDataset; Handler : TBusinessDataEvent );
    procedure SetMainDataset( Value : TDataset ); virtual;

    procedure SetupFields( Dataset : TDataset ); virtual;
    procedure CreateFields; virtual; abstract;
    function CreateMainField( const FieldName : string ) : TField;
    function CreateField( const FieldName : string;
                          Dataset : TDataset ) : TField;
    function CreateMainCalcField( const FieldName : string;
                                  FieldClass : TFieldClass;
                                  Size : Word ) : TField;
    function CreateCalcField( const FieldName : string;
                              FieldClass : TFieldClass;
                              Size : Word; Dataset : TDataset ) : TField;

    procedure HandleDataEvent( EventType : TDataEventType );
    procedure StartEdit; virtual;
    procedure FinishEdit; virtual;
    procedure StartInsert; virtual;
    procedure FinishInsert; virtual;
    procedure StartDelete; virtual;
    procedure FinishDelete; virtual;
    procedure StartPost; virtual;
    procedure FinishPost; virtual;
    procedure StartCancel; virtual;
    procedure FinishCancel; virtual;
    procedure CalculateFields; virtual;

    function GetErrorHandler : TRzErrorHandler; virtual;
    procedure SetErrorHandler( Value : TRzErrorHandler ); virtual;
  public
    procedure Validate; virtual;

    property SkipValidation : Boolean
      read FSkipValidation
      write FSkipValidation;

  published
    property MainDataset : TDataset
      read FMainDataset
      write SetMainDataset;

    property ErrorHandler : TRzErrorHandler
      read GetErrorHandler
      write SetErrorHandler;

    property IndexName : string
      read GetIndexName
      write SetIndexName;
  end;


procedure Register;


implementation

uses
  DBConsts, DBIErrs, Dialogs;



{==============================}
{== TRzBusinessTable Methods ==}
{==============================}

procedure TRzBusinessTable.DoStartEdit;
begin
  if Assigned( FOnStartEdit ) then
    FOnStartEdit( etStartEdit );
end;

procedure TRzBusinessTable.DoBeforeEdit;
begin
  inherited DoBeforeEdit;
  DoStartEdit;
end;

procedure TRzBusinessTable.DoAfterEdit;
begin
  DoFinishEdit;
  inherited DoAfterEdit;
end;

procedure TRzBusinessTable.DoFinishEdit;
begin
  if Assigned( FOnFinishEdit ) then
    FOnFinishEdit( etFinishEdit );
end;


procedure TRzBusinessTable.DoStartInsert;
begin
  if Assigned( FOnStartInsert ) then
    FOnStartInsert( etStartInsert );
end;

procedure TRzBusinessTable.DoBeforeInsert;
begin
  inherited DoBeforeInsert;
  DoStartInsert;
end;

procedure TRzBusinessTable.DoAfterInsert;
begin
  DoFinishInsert;
  inherited DoAfterInsert;
end;

procedure TRzBusinessTable.DoFinishInsert;
begin
  if Assigned( FOnFinishInsert ) then
    FOnFinishInsert( etFinishInsert );
end;


procedure TRzBusinessTable.DoStartDelete;
begin
  if Assigned( FOnStartDelete ) then
    FOnStartDelete( etStartDelete );
end;

procedure TRzBusinessTable.DoBeforeDelete;
begin
  inherited DoBeforeDelete;
  DoStartDelete;
end;

procedure TRzBusinessTable.DoAfterDelete;
begin
  DoFinishDelete;
  inherited DoAfterDelete;
end;

procedure TRzBusinessTable.DoFinishDelete;
begin
  if Assigned( FOnFinishDelete ) then
    FOnFinishDelete( etFinishDelete );
end;


procedure TRzBusinessTable.DoStartPost;
begin
  if Assigned( FOnStartPost ) then
    FOnStartPost( etStartPost );
end;

procedure TRzBusinessTable.DoBeforePost;
begin
  inherited DoBeforePost;
  DoStartPost;

  { During Post, Use our own Exception Handler }
  FOldExceptionHandler := Application.OnException;
  Application.OnException := ExceptionHandlerOnPost;
end;

procedure TRzBusinessTable.DoAfterPost;
begin
  Application.OnException := FOldExceptionHandler;
  DoFinishPost;
  inherited DoAfterPost;
end;

procedure TRzBusinessTable.DoFinishPost;
begin
  if Assigned( FOnFinishPost ) then
    FOnFinishPost( etFinishPost );
end;

procedure TRzBusinessTable.ExceptionHandlerOnPost( Sender : TObject; E : Exception );
begin
  if ( E is EDBEngineError ) and
     ( EDBEngineError( E ).Errors[ 0 ].ErrorCode = DBIERR_KEYVIOL ) then
  begin
    MessageDlg( 'Key Violation Error on Table ' + TableName, mtError, [ mbOK ], 0 );
    { If there was an exeption, then AfterPost won't get called.  Therefore,
      we need to re-establish original exception handler }
    Application.OnException := FOldExceptionHandler;

    { Handle Event processing on Key Violation }
    if Assigned( FOnKeyViolation ) then
      FOnKeyViolation( TableName );
  end
  else
    Application.ShowException( E );
end;


procedure TRzBusinessTable.DoStartCancel;
begin
  if Assigned( FOnStartCancel ) then
    FOnStartCancel( etStartCancel );
end;

procedure TRzBusinessTable.DoBeforeCancel;
begin
  inherited DoBeforeCancel;
  DoStartCancel;
end;

procedure TRzBusinessTable.DoAfterCancel;
begin
  DoFinishCancel;
  inherited DoAfterCancel;
end;

procedure TRzBusinessTable.DoFinishCancel;
begin
  if Assigned( FOnFinishCancel ) then
    FOnFinishCancel( etFinishCancel );
end;


procedure TRzBusinessTable.DoOnCalcFields;
begin
  inherited DoOnCalcFields;
  DoCalculateFields;
end;

procedure TRzBusinessTable.DoCalculateFields;
begin
  if Assigned( FOnCalculateFields ) then
    FOnCalculateFields( etCalculateFields );
end;



{==============================}
{== TRzBusinessQuery Methods ==}
{==============================}

procedure TRzBusinessQuery.DoStartEdit;
begin
  if Assigned( FOnStartEdit ) then
    FOnStartEdit( etStartEdit );
end;

procedure TRzBusinessQuery.DoBeforeEdit;
begin
  inherited DoBeforeEdit;
  DoStartEdit;
end;

procedure TRzBusinessQuery.DoAfterEdit;
begin
  DoFinishEdit;
  inherited DoAfterEdit;
end;

procedure TRzBusinessQuery.DoFinishEdit;
begin
  if Assigned( FOnFinishEdit ) then
    FOnFinishEdit( etFinishEdit );
end;


procedure TRzBusinessQuery.DoStartInsert;
begin
  if Assigned( FOnStartInsert ) then
    FOnStartInsert( etStartInsert );
end;

procedure TRzBusinessQuery.DoBeforeInsert;
begin
  inherited DoBeforeInsert;
  DoStartInsert;
end;

procedure TRzBusinessQuery.DoAfterInsert;
begin
  DoFinishInsert;
  inherited DoAfterInsert;
end;

procedure TRzBusinessQuery.DoFinishInsert;
begin
  if Assigned( FOnFinishInsert ) then
    FOnFinishInsert( etFinishInsert );
end;


procedure TRzBusinessQuery.DoStartDelete;
begin
  if Assigned( FOnStartDelete ) then
    FOnStartDelete( etStartDelete );
end;

procedure TRzBusinessQuery.DoBeforeDelete;
begin
  inherited DoBeforeDelete;
  DoStartDelete;
end;

procedure TRzBusinessQuery.DoAfterDelete;
begin
  DoFinishDelete;
  inherited DoAfterDelete;
end;

procedure TRzBusinessQuery.DoFinishDelete;
begin
  if Assigned( FOnFinishDelete ) then
    FOnFinishDelete( etFinishDelete );
end;


procedure TRzBusinessQuery.DoStartPost;
begin
  if Assigned( FOnStartPost ) then
    FOnStartPost( etStartPost );
end;

procedure TRzBusinessQuery.DoBeforePost;
begin
  inherited DoBeforePost;
  DoStartPost;

  { During Post, Use our own Exception Handler }
  FOldExceptionHandler := Application.OnException;
  Application.OnException := ExceptionHandlerOnPost;
end;

procedure TRzBusinessQuery.DoAfterPost;
begin
  Application.OnException := FOldExceptionHandler;
  DoFinishPost;
  inherited DoAfterPost;
end;

procedure TRzBusinessQuery.DoFinishPost;
begin
  if Assigned( FOnFinishPost ) then
    FOnFinishPost( etFinishPost );
end;

procedure TRzBusinessQuery.ExceptionHandlerOnPost( Sender : TObject; E : Exception );
begin
  if ( E is EDBEngineError ) and
     ( EDBEngineError( E ).Errors[ 0 ].ErrorCode = DBIERR_KEYVIOL ) then
  begin
    MessageDlg( 'Key Violation Error on Query', mtError, [ mbOK ], 0 );
    { If there was an exeption, then AfterPost won't get called.  Therefore,
      we need to re-establish original exception handler }
    Application.OnException := FOldExceptionHandler;

    { Handle Event processing on Key Violation }
    if Assigned( FOnKeyViolation ) then
      FOnKeyViolation( 'Query' );
  end
  else
    Application.ShowException( E );
end;


procedure TRzBusinessQuery.DoStartCancel;
begin
  if Assigned( FOnStartCancel ) then
    FOnStartCancel( etStartCancel );
end;

procedure TRzBusinessQuery.DoBeforeCancel;
begin
  inherited DoBeforeCancel;
  DoStartCancel;
end;

procedure TRzBusinessQuery.DoAfterCancel;
begin
  DoFinishCancel;
  inherited DoAfterCancel;
end;

procedure TRzBusinessQuery.DoFinishCancel;
begin
  if Assigned( FOnFinishCancel ) then
    FOnFinishCancel( etFinishCancel );
end;


procedure TRzBusinessQuery.DoOnCalcFields;
begin
  inherited DoOnCalcFields;
  DoCalculateFields;
end;

procedure TRzBusinessQuery.DoCalculateFields;
begin
  if Assigned( FOnCalculateFields ) then
    FOnCalculateFields( etCalculateFields );
end;





{==================================}
{== TRzBusinessComponent Methods ==}
{==================================}


{= TRzBusinessComponent.Notification                                          =}
{=   This method is overridden to ensure that FDataset is set to nil if the   =}
{=   corresponding Dataset is deleted from the form. This method should be    =}
{=   used whenever a contains a reference to another component.               =}

procedure TRzBusinessComponent.Notification( AComponent : TComponent;
                                             Operation : TOperation );
begin
  inherited Notification( AComponent, Operation );
  if Operation = opRemove then
  begin
    if AComponent = FMainDataset then
      FMainDataset := nil
    else if AComponent = FErrorHandler then
      FErrorHandler := nil;
  end;
end;


{===============================================================================
  SetEmbeddedObjectDataset                                                

  This method is overridden by descendant classes who have embedded
  business objects that use the same dataset as the business object that
  owns them. Inside this method, a descendant class sets the embedded
  object's MainDataset to the inherited MainDataset.
===============================================================================}

procedure TRzBusinessComponent.SetEmbeddedObjectsDataset;
begin
end;


procedure TRzBusinessComponent.LinkDataEvents( Dataset : TDataset;
                                               Handler : TBusinessDataEvent );
begin
  if Dataset = nil then
    Exit;                              { Nothing to link to if FDataset is nil }

  if Dataset is TRzBusinessTable then
  begin
    with Dataset as TRzBusinessTable do
    begin
      OnStartEdit := Handler;
      OnFinishEdit := Handler;
      OnStartInsert := Handler;
      OnFinishInsert := Handler;
      OnStartDelete := Handler;
      OnFinishDelete := Handler;
      OnStartPost := Handler;
      OnFinishPost := Handler;
      OnStartCancel := Handler;
      OnFinishCancel := Handler;
      OnCalculateFields := Handler;
    end;
  end
  else if Dataset is TRzBusinessQuery then
  begin
    with Dataset as TRzBusinessQuery do
    begin
      OnStartEdit := Handler;
      OnFinishEdit := Handler;
      OnStartInsert := Handler;
      OnFinishInsert := Handler;
      OnStartDelete := Handler;
      OnFinishDelete := Handler;
      OnStartPost := Handler;
      OnFinishPost := Handler;
      OnStartCancel := Handler;
      OnFinishCancel := Handler;
      OnCalculateFields := Handler;
    end;
  end;
end; {= TRzBusinessComponent.LinkDataEvents =}


{= TRzBusinessComponent.SetMainDataset                                        =}
{=   After setting the FMainDataset, you must call SetupFields to ensure that =}
{=   the  field objects get created.                                          =}

procedure TRzBusinessComponent.SetMainDataset( Value : TDataset );
begin
  if FMainDataset <> Value then
  begin
    FMainDataset := Value;
    try
      LinkDataEvents( FMainDataset, HandleDataEvent );
      SetupFields( FMainDataset );
      SetEmbeddedObjectsDataset;
    except
      on EDBEngineError do
      begin
        MessageDlg( 'To connect to a Dataset, the Dataset must be able to ' +
                    'generate a result set.', mtError, [ mbOk ], 0 );
        FMainDataset := nil;
      end;

      on EDatabaseError do
      begin
        FMainDataset := nil;
        raise;
      end;
    end;
  end;
end; {= TRzBusinessComponent.SetMainDataset =}



{= TRzBusinessComponent.SetupFields                                           =}
{=   This method is responsible for making sure the appropriate functions are =}
{=   called when the field objects are created. More precisely, there is pre- =}
{=   and post- operations that must be performed.                             =}
{=                                                                            =}
{=   Pre-Operations   Dataset must be closed                                  =}
{=                    Dataset's FieldDefs must be updated                     =}
{=                                                                            =}
{=   Post-Operations  Re-establish original Dataset state                     =}
{=                    Send DataEvent message to Dataset.Designer              =}

procedure TRzBusinessComponent.SetupFields( Dataset : TDataset );
var
  ActiveState : Boolean;
begin
  if Dataset = nil then
    Exit;                                 { No need to setup if Dataset is nil }

  { Dataset must be closed in order to create new field objects }
  ActiveState := Dataset.Active;                          { Save current state }
  if ActiveState then
    Dataset.Active := False;

  { Must call Update to gain Access to All Field Defs }
  Dataset.FieldDefs.Update;

  CreateFields;                     { Call to Descendant's CreateFields method }

  Dataset.Active := ActiveState;

  { If the developer has not yet invoked the Field Editor, then the Dataset    }
  { Designer does not exist. If the designer doesn't exist, it is not          }
  { necessary to call DataEvent.  Besides if you do, it will cause a GPF       }

  if Dataset.Designer <> nil then
    Dataset.Designer.DataEvent( deFieldListChange, 0 );
end; {= TRzBusinessComponent.SetupFields =}



{= TRzBusinessComponent.CreateField                                           =}
{=   This method simplifies the process of creating fields.  First, the       =}
{=   Dataset is searched for a matching FieldName. If one exists, then that   =}
{=   one is used.  If not, then a  new field is created from the FieldDef     =}
{=   object corresponding to the desired field name.                          =}

function TRzBusinessComponent.CreateMainField( const FieldName : string ) : TField;
begin
  Result := CreateField( FieldName, FMainDataset );
end;

function TRzBusinessComponent.CreateField( const FieldName : string;
                                           Dataset : TDataset ) : TField;
begin
  { First, try to find an existing field object.  FindField is the same as     }
  { FieldByName, but doesn't raise an exception if the field cannot be found.  }
  Result := Dataset.FindField( FieldName );

  if Result = nil then
  begin
    { If cannot find an existing field object... }
    { Instruct the FieldDefs object create a Field Object }

    Result := Dataset.FieldDefs.Find( FieldName ).CreateField( Owner );

    { We need to give the new field object a name so that it may appear in }
    { the Object Inspector. Use the default naming convention. }

    Result.Name := Dataset.Name + FieldName;
  end;
end; {= TRzBusinessComponent.CreateField =}


{= TRzBusinessComponent.CreateCalcField                                       =}
{=   This method simplifies the process of creating calculated fields. First, =}
{=   the Dataset is searched for a matching FieldName. If one exists, then    =}
{=   that one is used.  If not, then a new field is created from the passed   =}
{=   FieldClass parameter.                                                    =}
function TRzBusinessComponent.CreateMainCalcField( const FieldName : string;
                                                   FieldClass : TFieldClass;
                                                   Size : Word ) : TField;
begin
  Result := CreateCalcField( FieldName, FieldClass, Size, FMainDataset );
end;

function TRzBusinessComponent.CreateCalcField( const FieldName : string;
                                               FieldClass : TFieldClass;
                                               Size : Word;
                                               Dataset : TDataset ) : TField;
begin
  Result := Dataset.FindField( FieldName );
  if Result = nil then
  begin
    if FieldClass = nil then
      DBErrorFmt( SUnknownFieldType, [ FieldName ] );

    { Create Desired Field Object using a Class Reference Variable }
    Result := FieldClass.Create( Owner );
    try
      Result.FieldName := FieldName;

      { Size is only necessary if new field is one of the following types }
      if ( Result is TStringField ) or
         ( Result is TBCDField ) or
         ( Result is TBlobField ) or
         ( Result is TBytesField ) or
         ( Result is TVarBytesField ) then
      begin
        Result.Size := Size;
      end;
      Result.Calculated := True;
      Result.Dataset := Dataset;
      Result.Name := Dataset.Name + FieldName;
    except
      Result.Free;                { If error occurs, be sure to release memory }
      raise;
    end;
  end;
end; {= TRzBusinessComponent.CreateCalcField =}



procedure TRzBusinessComponent.HandleDataEvent( EventType : TDataEventType );
begin
  case EventType of
    etStartEdit: StartEdit;
    etFinishEdit: FinishEdit;
    etStartInsert: StartInsert;
    etFinishInsert: FinishInsert;
    etStartDelete: StartDelete;
    etFinishDelete: FinishDelete;
    etStartPost: StartPost;
    etFinishPost: FinishPost;
    etStartCancel: StartCancel;
    etFinishCancel: FinishCancel;
    etCalculateFields: CalculateFields;
  end;
end;


procedure TRzBusinessComponent.StartEdit;
begin
end;

procedure TRzBusinessComponent.FinishEdit;
begin
end;


procedure TRzBusinessComponent.StartInsert;
begin
end;

procedure TRzBusinessComponent.FinishInsert;
begin
end;


procedure TRzBusinessComponent.StartDelete;
begin
end;

procedure TRzBusinessComponent.FinishDelete;
begin
end;


procedure TRzBusinessComponent.StartPost;
begin
  if not FSkipValidation then
  begin
    if FErrorHandler <> nil then
      FErrorHandler.Clear;

    Validate;                                             { Perform Validation }

    if FErrorHandler <> nil then
    begin
      if FErrorHandler.HandleErrors then
      begin
        { At this point, errors or warnings have been displayed to the user,
          and we must decide whether or not to abort the post }
        if FErrorHandler.UserResponse <> mrProceed then
          raise EValidationError.Create( 'Validation Error Occurred-Abort Post' );
      end;
    end;
  end;
end;

procedure TRzBusinessComponent.Validate;
begin
  {= Descendant classes override this method to perform validation on the     =}
  {= record right before it is about to be posted.  Validation errors are     =}
  {= added to the ErrorHandler which gets processed in the StartPost method.  =}
end;

procedure TRzBusinessComponent.FinishPost;
begin
end;


procedure TRzBusinessComponent.StartCancel;
begin
end;

procedure TRzBusinessComponent.FinishCancel;
begin
end;


procedure TRzBusinessComponent.CalculateFields;
begin
end;


function TRzBusinessComponent.GetIndexName : string;
begin
  if ( FMainDataset <> nil ) and ( FMainDataset is TTable ) then
    Result := TTable( FMainDataset ).IndexName
  else
    Result := '';
end;


procedure TRzBusinessComponent.SetIndexName( const Value : string );
begin
  if FMainDataset is TTable then
  begin
    if TTable( FMainDataset ).IndexName <> Value then
        TTable( FMainDataset ).IndexName := Value;
  end;
end;



function  TRzBusinessComponent.GetErrorHandler: TRzErrorHandler;
begin
  Result := FErrorHandler;
end;


procedure TRzBusinessComponent.SetErrorHandler( Value : TRzErrorHandler );
begin
  if FErrorHandler <> Value then
    FErrorHandler := Value;
end;



{========================}
{== Register Procedure ==}
{========================}

procedure Register;
begin
  RegisterComponents( 'RzData', [ TRzBusinessTable, TRzBusinessQuery ] );
end;

end.
